// download_list.h  (this is -*-c++-*-)
//
//  Copyright 2001 Daniel Burrows
//
//  A better download manager.  (..finally..)

#ifndef DOWNLOAD_LIST_H
#define DOWNLOAD_LIST_H

#include <vector>
#include <string>

#include <apt-pkg/acquire.h>

#include <vscreen/vscreen_widget.h>
#include <vscreen/slotarg.h>

class download_manager;

class download_list:public vscreen_widget
{
  // Caches the last-seen info about a worker.
  struct workerinf
  {
    std::wstring msg;
    int current,total;

    workerinf(const std::wstring &_msg, int _current, int _total);
    workerinf(const std::string &_msg, int _current, int _total);
  };

  typedef std::pair<std::wstring, style> msg;

  // Contains strings paired with attributes.
  typedef std::vector<msg> msglist;

  // Contains strings paired with the current and total sizes associated
  // with that worker (updated in Pulse)
  typedef std::vector<workerinf> workerlist;

  msglist msgs;
  workerlist workers;

  // Where in the list we are (can be >msgs.size() -- that would mean we're
  // viewing the currently progressing downloads)
  unsigned int start;

  // Whether we should stick to the bottom of the screen (set to false
  // when the user scrolls up, set back to true when the user scrolls
  // back down)
  bool sticky_end;

  // Set to true if the user asks to cancel the download
  bool cancelled;

  // Set to true if an item failed.
  bool failed;

  slot0arg abortslot;

  // Will we display things other than the current workers?
  bool display_messages;

  // The starting offset to display messages with.
  std::string::size_type startx;

  // We have to mirror the download_manager's information (ewww)
  // Another option is to store a reference to the download-manager,
  // but I don't like that for other reasons. (sigh..)
  //
  // A note: these values are only updated when Pulse() is called anyway.
  unsigned long TotalItems, CurrentItems;
  double CurrentCPS, TotalBytes, CurrentBytes;

  // Updates the set of displayed progress bars.
  void update_workers(pkgAcquire *Owner);

  // Syncs the current location in the list which is displayed
  void sync_top();

  // Cancels a download
  void cancel() {cancelled=true;}

  void layout_me();
protected:
  void paint(const style &st);
  bool handle_key(const key &k);
public:
  download_list(slot0arg _abortslot=NULL,
		bool _display_messages=true);

  bool MediaChange(std::string media, std::string drive,
		   download_manager &manager);
  void IMSHit(pkgAcquire::ItemDesc &itmdesc, download_manager &manager);
  void Fetch(pkgAcquire::ItemDesc &itmdesc, download_manager &manager);
  void Done(pkgAcquire::ItemDesc &itmdesc, download_manager &manager);
  void Fail(pkgAcquire::ItemDesc &itmdesc, download_manager &manager);
  bool Pulse(pkgAcquire *Owner, download_manager &manager);
  void Start(download_manager &manager);
  void Stop(download_manager &manager);
  void Complete(download_manager &manager);

  int width_request();
  int height_request(int w);

  bool get_cursorvisible() {return false;}
  point get_cursorloc() {return point(0,0);}
  bool focus_me() {return true;}

  // FIXME: overriding this is a terrible hack.  A cancel() hook should be
  // used instead.
  void destroy();

  // Used to move around in the list (can be called manually to
  // simulate these actions)
  void pageup();
  void pagedown();
  void lineup();
  void linedown();
  void skip_to_top();
  void skip_to_bottom();
  void shift_left();
  void shift_right();
};

#endif
