// view_changelog.cc
//
//   Copyright (C) 2004-2005 Daniel Burrows
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU General Public License as
//   published by the Free Software Foundation; either version 2 of
//   the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; see the file COPYING.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.

#include <vscreen/config/colors.h>
#include <vscreen/vs_pager.h>
#include <vscreen/vs_scrollbar.h>
#include <vscreen/vs_table.h>

#include "download_bar.h"
#include "download_manager.h"
#include "ui.h"

#include <generic/apt.h>
#include <generic/pkg_changelog.h>
#include <generic/config_signal.h>

#include <sigc++/functors/mem_fun.h>

using namespace std;

class pkg_changelog_screen:public vs_file_pager
// Needed to make sure the changelog is properly cleaned up.
{
  pkg_changelog *cl;
public:
  pkg_changelog_screen(pkg_changelog *_cl, vscreen_widget *owner=NULL, int x=0, int y=0, int width=0, int height=0):
    vs_file_pager(_cl->get_filename()),cl(_cl)
  {
  }

  virtual ~pkg_changelog_screen() {delete cl;}
};

void view_changelog(pkgCache::VerIterator ver)
{
  bool in_debian=false;

  // TODO: add a configurable association between origins and changelog URLs.
  for(pkgCache::VerFileIterator vf=ver.FileList();
      !vf.end() && !in_debian; ++vf)
    if(!vf.File().end() && vf.File().Origin()!=NULL &&
       strcmp(vf.File().Origin(), "Debian")==0)
      in_debian=true;

  if(!in_debian)
    {
      show_message(_("You can only view changelogs of official Debian packages."),
		   NULL, get_style("Error"));
      return;
    }

  download_manager *widget=gen_download_progress(true,
						 _("Downloading Changelog"),
						 "",
						 _("Download Changelog"),
						 NULL);
  pkg_changelog *cl=get_changelog(ver, widget,
				  aptcfg->FindI(PACKAGE "::UI::Download-Poll-Interval", 50000));
  widget->Complete();
  if(cl)
    {
      char buf[512];
      snprintf(buf, 512, _("ChangeLog of %s"), ver.ParentPkg().Name());
      string menulabel(buf);
      snprintf(buf, 512, _("%s changes"), ver.ParentPkg().Name());
      string tablabel(buf);

      pkg_changelog_screen *cs=new pkg_changelog_screen(cl);
      vs_table *t=new vs_table;
      vs_scrollbar *s=new vs_scrollbar(vs_scrollbar::VERTICAL);

      cs->line_changed.connect(sigc::mem_fun(*s, &vs_scrollbar::set_slider));
      s->scrollbar_interaction.connect(sigc::mem_fun(*cs, &pkg_changelog_screen::scroll_page));
      cs->scroll_top();

      t->add_widget_opts(cs, 0, 0, 1, 1, vs_table::EXPAND|vs_table::SHRINK, vs_table::EXPAND);
      t->add_widget_opts(s, 0, 1, 1, 1, 0, vs_table::EXPAND);

      t->show_all();

      insert_main_widget(t, menulabel, "", tablabel);
    }
}
